@@ -22,9 +22,11 @@
  * @file gnunet_chat_context_intern.c
  */
 
+#include "gnunet_chat_discourse.h"
 #include "gnunet_chat_invitation.h"
 #include "gnunet_chat_message.h"
-#include "gnunet_chat_tagging.h"
+
+#include "internal/gnunet_chat_tagging.h"
 
 #include <gnunet/gnunet_common.h>
 #include <gnunet/gnunet_error_codes.h>
@@ -63,8 +65,8 @@ it_destroy_context_taggings (GNUNET_UNUSED void *cls,
 {
   GNUNET_assert(value);
 
-  struct GNUNET_CHAT_Tagging *tagging = value;
-  tagging_destroy(tagging);
+  struct GNUNET_CHAT_InternalTagging *tagging = value;
+  internal_tagging_destroy(tagging);
   return GNUNET_YES;
 }
 
@@ -80,6 +82,18 @@ it_destroy_context_invites (GNUNET_UNUSED void *cls,
   return GNUNET_YES;
 }
 
+enum GNUNET_GenericReturnValue
+it_destroy_context_discourses (GNUNET_UNUSED void *cls,
+                               GNUNET_UNUSED const struct GNUNET_ShortHashCode *key,
+                               void *value)
+{
+  GNUNET_assert(value);
+
+  struct GNUNET_CHAT_Discourse *discourse = value;
+  discourse_destroy(discourse);
+  return GNUNET_YES;
+}
+
 enum GNUNET_GenericReturnValue
 it_iterate_context_requests (void *cls,
                              const struct GNUNET_HashCode *key,
@@ -104,15 +118,7 @@ cb_context_request_messages (void *cls)
   context->request_task = NULL;
 
   if (!(context->room))
-  {
-    context->request_task = GNUNET_SCHEDULER_add_with_priority(
-      GNUNET_SCHEDULER_PRIORITY_IDLE,
-      cb_context_request_messages,
-      context
-    );
-
     return;
-  }
 
   GNUNET_CONTAINER_multihashmap_iterate(
     context->requests,
@@ -139,6 +145,7 @@ cont_context_write_records (void *cls,
       NULL,
       context,
       GNUNET_CHAT_FLAG_WARNING,
-      GNUNET_ErrorCode_get_hint(ec)
+      GNUNET_ErrorCode_get_hint(ec),
+      GNUNET_YES
     );
 }
