@@ -111,7 +111,8 @@ GNUNET_CHAT_account_create (struct GNUNET_CHAT_Handle *handle,
 
   char *low = util_get_lower(name);
 
-  int result = handle_create_account(handle, low);
+  enum GNUNET_GenericReturnValue result;
+  result = handle_create_account(handle, low);
 
   GNUNET_free(low);
   return result;
@@ -120,19 +121,25 @@ GNUNET_CHAT_account_create (struct GNUNET_CHAT_Handle *handle,
 
 enum GNUNET_GenericReturnValue
 GNUNET_CHAT_account_delete(struct GNUNET_CHAT_Handle *handle,
-			                     const char* name)
+			                     const char *name)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
   if ((!handle) || (handle->destruction) || (!name))
     return GNUNET_SYSERR;
 
-  return handle_delete_account(handle, name);
+  const struct GNUNET_CHAT_Account *account;
+  account = handle_get_account_by_name(handle, name, GNUNET_NO);
+
+  if (!account)
+    return GNUNET_SYSERR;
+
+  return handle_delete_account(handle, account);
 }
 
 
 int
-GNUNET_CHAT_iterate_accounts (const struct GNUNET_CHAT_Handle *handle,
+GNUNET_CHAT_iterate_accounts (struct GNUNET_CHAT_Handle *handle,
                               GNUNET_CHAT_AccountCallback callback,
                               void *cls)
 {
@@ -141,7 +148,7 @@ GNUNET_CHAT_iterate_accounts (const struct GNUNET_CHAT_Handle *handle,
   if ((!handle) || (handle->destruction))
     return GNUNET_SYSERR;
 
-  int result = 0;
+  int iterations = 0;
 
   struct GNUNET_CHAT_InternalAccounts *accounts = handle->accounts_head;
   while (accounts)
@@ -149,7 +156,7 @@ GNUNET_CHAT_iterate_accounts (const struct GNUNET_CHAT_Handle *handle,
     if ((!(accounts->account)) || (accounts->op))
       goto skip_account;
 
-    result++;
+    iterations++;
 
     if ((callback) && (GNUNET_YES != callback(cls, handle, accounts->account)))
       break;
@@ -158,7 +165,7 @@ GNUNET_CHAT_iterate_accounts (const struct GNUNET_CHAT_Handle *handle,
     accounts = accounts->next;
   }
 
-  return result;
+  return iterations;
 }
 
 
@@ -171,26 +178,13 @@ GNUNET_CHAT_find_account (const struct GNUNET_CHAT_Handle *handle,
   if ((!handle) || (handle->destruction))
     return NULL;
 
-  struct GNUNET_CHAT_InternalAccounts *accounts = handle->accounts_head;
-  while (accounts)
-  {
-    if ((!(accounts->account)) || (accounts->op))
-      goto skip_account;
-
-    if (0 == strcmp(accounts->account->name, name))
-      return accounts->account;
-
-  skip_account:
-    accounts = accounts->next;
-  }
-
-  return NULL;
+  return handle_get_account_by_name(handle, name, GNUNET_YES);
 }
 
 
 void
 GNUNET_CHAT_connect (struct GNUNET_CHAT_Handle *handle,
-                     const struct GNUNET_CHAT_Account *account)
+                     struct GNUNET_CHAT_Account *account)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
@@ -248,7 +242,7 @@ GNUNET_CHAT_disconnect (struct GNUNET_CHAT_Handle *handle)
 }
 
 
-const struct GNUNET_CHAT_Account*
+struct GNUNET_CHAT_Account*
 GNUNET_CHAT_get_connected (const struct GNUNET_CHAT_Handle *handle)
 {
   GNUNET_CHAT_VERSION_ASSERT();
@@ -285,10 +279,10 @@ GNUNET_CHAT_set_name (struct GNUNET_CHAT_Handle *handle,
     return GNUNET_NO;
 
   char *low = util_get_lower(name);
-  int result;
+  enum GNUNET_GenericReturnValue result;
 
   if (handle->current)
-    result = handle_rename_account(handle, handle->current->name, low);
+    result = handle_rename_account(handle, handle->current, low);
   else
     result = GNUNET_OK;
 
@@ -329,8 +323,7 @@ GNUNET_CHAT_get_key (const struct GNUNET_CHAT_Handle *handle)
 void
 GNUNET_CHAT_set_attribute (struct GNUNET_CHAT_Handle *handle,
                            const char *name,
-                           const char *value,
-                           struct GNUNET_TIME_Relative expires)
+                           const char *value)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
@@ -344,8 +337,11 @@ GNUNET_CHAT_set_attribute (struct GNUNET_CHAT_Handle *handle,
   if ((!key) || (!name))
     return;
 
+  struct GNUNET_TIME_Relative rel;
+  rel = GNUNET_TIME_relative_get_forever_();
+
   struct GNUNET_CHAT_AttributeProcess *attributes;
-  attributes = internal_attributes_create_store(handle, name, expires);
+  attributes = internal_attributes_create_store(handle, name, rel);
 
   if (!attributes)
     return;
@@ -624,7 +620,7 @@ GNUNET_CHAT_uri_destroy (struct GNUNET_CHAT_Uri *uri)
 
 struct GNUNET_CHAT_Lobby*
 GNUNET_CHAT_lobby_open (struct GNUNET_CHAT_Handle *handle,
-                        struct GNUNET_TIME_Relative delay,
+                        unsigned int delay,
                         GNUNET_CHAT_LobbyCallback callback,
                         void *cls)
 {
@@ -633,6 +629,10 @@ GNUNET_CHAT_lobby_open (struct GNUNET_CHAT_Handle *handle,
   if ((!handle) || (handle->destruction))
     return NULL;
 
+  struct GNUNET_TIME_Relative rel = GNUNET_TIME_relative_multiply(
+    GNUNET_TIME_relative_get_second_(), delay
+  );
+
   struct GNUNET_CHAT_InternalLobbies *lobbies = GNUNET_new(
     struct GNUNET_CHAT_InternalLobbies
   );
@@ -645,7 +645,7 @@ GNUNET_CHAT_lobby_open (struct GNUNET_CHAT_Handle *handle,
     lobbies
   );
 
-  lobby_open(lobbies->lobby, delay, callback, cls);
+  lobby_open(lobbies->lobby, rel, callback, cls);
 
   return lobbies->lobby;
 }
@@ -941,13 +941,13 @@ GNUNET_CHAT_account_get_name (const struct GNUNET_CHAT_Account *account)
   if (!account)
     return NULL;
 
-  return account->name;
+  return account_get_name(account);
 }
 
 
 void
 GNUNET_CHAT_account_get_attributes (struct GNUNET_CHAT_Handle *handle,
-                                    const struct GNUNET_CHAT_Account *account,
+                                    struct GNUNET_CHAT_Account *account,
                                     GNUNET_CHAT_AccountAttributeCallback callback,
                                     void *cls)
 {
@@ -1320,6 +1320,25 @@ GNUNET_CHAT_contact_is_tagged (const struct GNUNET_CHAT_Contact *contact,
 }
 
 
+int
+GNUNET_CHAT_contact_iterate_tags (struct GNUNET_CHAT_Contact *contact,
+                                  GNUNET_CHAT_ContactTagCallback callback,
+                                  void *cls)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if (!contact)
+    return GNUNET_SYSERR;
+
+  return contact_iterate_tags(
+    contact,
+    NULL,
+    callback,
+    cls
+  );
+}
+
+
 void
 GNUNET_CHAT_contact_get_attributes (struct GNUNET_CHAT_Contact *contact,
                                     GNUNET_CHAT_ContactAttributeCallback callback,
@@ -1420,14 +1439,14 @@ GNUNET_CHAT_group_get_user_pointer (const struct GNUNET_CHAT_Group *group)
 }
 
 
-void
-GNUNET_CHAT_group_invite_contact (const struct GNUNET_CHAT_Group *group,
+enum GNUNET_GenericReturnValue
+GNUNET_CHAT_group_invite_contact (struct GNUNET_CHAT_Group *group,
 				                          struct GNUNET_CHAT_Contact *contact)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
   if ((!group) || (!contact) || (!contact->member))
-    return;
+    return GNUNET_SYSERR;
 
   struct GNUNET_CHAT_Context *context = contact_find_context(
     contact,
@@ -1435,7 +1454,7 @@ GNUNET_CHAT_group_invite_contact (const struct GNUNET_CHAT_Group *group,
   );
 
   if (!context)
-    return;
+    return GNUNET_SYSERR;
 
   const struct GNUNET_HashCode *key = GNUNET_MESSENGER_room_get_key(
     group->context->room
@@ -1453,11 +1472,12 @@ GNUNET_CHAT_group_invite_contact (const struct GNUNET_CHAT_Group *group,
   GNUNET_memcpy(&(msg.body.invite.key), key, sizeof(msg.body.invite.key));
 
   GNUNET_MESSENGER_send_message(context->room, &msg, contact->member);
+  return GNUNET_OK;
 }
 
 
 int
-GNUNET_CHAT_group_iterate_contacts (const struct GNUNET_CHAT_Group *group,
+GNUNET_CHAT_group_iterate_contacts (struct GNUNET_CHAT_Group *group,
                                     GNUNET_CHAT_GroupContactCallback callback,
                                     void *cls)
 {
@@ -1546,25 +1566,27 @@ GNUNET_CHAT_context_get_status (const struct GNUNET_CHAT_Context *context)
 }
 
 
-void
+enum GNUNET_GenericReturnValue
 GNUNET_CHAT_context_request (struct GNUNET_CHAT_Context *context)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
-  if ((!context) || (context->room))
-    return;
+  if (!context)
+    return GNUNET_SYSERR;
+  else if (context->room)
+    return GNUNET_OK;
 
   struct GNUNET_CHAT_Handle *handle = context->handle;
 
   if ((!handle) || (!(context->contact)))
-    return;
+    return GNUNET_SYSERR;
 
   struct GNUNET_CHAT_Contact *contact = contact_create_from_member(
     handle, context->contact
   );
 
   if (!contact)
-    return;
+    return GNUNET_SYSERR;
 
   context->type = GNUNET_CHAT_CONTEXT_TYPE_CONTACT;
 
@@ -1607,13 +1629,14 @@ GNUNET_CHAT_context_request (struct GNUNET_CHAT_Context *context)
   GNUNET_memcpy(&(msg.body.invite.key), &key, sizeof(msg.body.invite.key));
 
   GNUNET_MESSENGER_send_message(other->room, &msg, context->contact);
-  return;
+  return GNUNET_OK;
 
 cleanup_room:
   GNUNET_MESSENGER_close_room(room);
 
 cleanup_contact:
   contact_destroy(contact);
+  return GNUNET_SYSERR;
 }
 
 
@@ -1711,7 +1734,7 @@ GNUNET_CHAT_context_send_text (struct GNUNET_CHAT_Context *context,
 
 enum GNUNET_GenericReturnValue
 GNUNET_CHAT_context_send_read_receipt (struct GNUNET_CHAT_Context *context,
-				                               const struct GNUNET_CHAT_Message *message)
+				                               struct GNUNET_CHAT_Message *message)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
@@ -1861,7 +1884,7 @@ file_binding:
 
 enum GNUNET_GenericReturnValue
 GNUNET_CHAT_context_share_file (struct GNUNET_CHAT_Context *context,
-				                        const struct GNUNET_CHAT_File *file)
+				                        struct GNUNET_CHAT_File *file)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
@@ -1894,7 +1917,7 @@ GNUNET_CHAT_context_share_file (struct GNUNET_CHAT_Context *context,
 
 enum GNUNET_GenericReturnValue
 GNUNET_CHAT_context_send_tag (struct GNUNET_CHAT_Context *context,
-                              const struct GNUNET_CHAT_Message *message,
+                              struct GNUNET_CHAT_Message *message,
                               const char *tag)
 {
   GNUNET_CHAT_VERSION_ASSERT();
@@ -1925,15 +1948,18 @@ GNUNET_CHAT_context_send_tag (struct GNUNET_CHAT_Context *context,
 
 struct GNUNET_CHAT_Discourse*
 GNUNET_CHAT_context_open_discourse (struct GNUNET_CHAT_Context *context,
-                                    const struct GNUNET_ShortHashCode *id)
+                                    const struct GNUNET_CHAT_DiscourseId *id)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
   if ((!context) || (!(context->discourses)) || (!(context->room)) || (!id))
     return NULL;
 
+  struct GNUNET_ShortHashCode sid;
+  util_shorthash_from_discourse_id(id, &sid);
+
   struct GNUNET_CHAT_Discourse *discourse = GNUNET_CONTAINER_multishortmap_get(
-    context->discourses, id
+    context->discourses, &sid
   );
 
   if (!discourse)
@@ -1941,7 +1967,7 @@ GNUNET_CHAT_context_open_discourse (struct GNUNET_CHAT_Context *context,
     discourse = discourse_create(context, id);
 
     if (GNUNET_OK != GNUNET_CONTAINER_multishortmap_put(context->discourses,
-        id, discourse, GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
+        &sid, discourse, GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
     {
       discourse_destroy(discourse);
       return NULL;
@@ -1952,7 +1978,11 @@ GNUNET_CHAT_context_open_discourse (struct GNUNET_CHAT_Context *context,
   memset(&msg, 0, sizeof(msg));
 
   msg.header.kind = GNUNET_MESSENGER_KIND_SUBSCRIBE;
-  GNUNET_memcpy(&(msg.body.subscribe.discourse), id, sizeof(struct GNUNET_ShortHashCode));
+  GNUNET_memcpy(
+    &(msg.body.subscribe.discourse),
+    &sid,
+    sizeof(struct GNUNET_ShortHashCode)
+  );
 
   const struct GNUNET_TIME_Relative subscribtion_time = GNUNET_TIME_relative_multiply(
     GNUNET_TIME_relative_get_second_(), 10
@@ -2054,15 +2084,23 @@ GNUNET_CHAT_message_get_kind (const struct GNUNET_CHAT_Message *message)
 }
 
 
-struct GNUNET_TIME_Absolute
+time_t
 GNUNET_CHAT_message_get_timestamp (const struct GNUNET_CHAT_Message *message)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
   if ((!message) || (GNUNET_YES != message_has_msg(message)))
-    return GNUNET_TIME_absolute_get_zero_();
+    return ((time_t) -1);
 
-  return GNUNET_TIME_absolute_ntoh(message->msg->header.timestamp);
+  struct GNUNET_TIME_Absolute abs = GNUNET_TIME_absolute_ntoh(
+    message->msg->header.timestamp
+  );
+
+  struct GNUNET_TIME_Timestamp ts = GNUNET_TIME_absolute_to_timestamp(
+    abs
+  );
+
+  return (time_t) GNUNET_TIME_timestamp_to_s(ts);
 }
 
 
@@ -2206,7 +2244,7 @@ GNUNET_CHAT_message_is_tagged (const struct GNUNET_CHAT_Message *message,
 
 
 int
-GNUNET_CHAT_message_get_read_receipt (const struct GNUNET_CHAT_Message *message,
+GNUNET_CHAT_message_get_read_receipt (struct GNUNET_CHAT_Message *message,
                                       GNUNET_CHAT_MessageReadReceiptCallback callback,
                                       void *cls)
 {
@@ -2254,7 +2292,7 @@ GNUNET_CHAT_message_get_text (const struct GNUNET_CHAT_Message *message)
 }
 
 
-const struct GNUNET_CHAT_Account*
+struct GNUNET_CHAT_Account*
 GNUNET_CHAT_message_get_account (const struct GNUNET_CHAT_Message *message)
 {
   GNUNET_CHAT_VERSION_ASSERT();
@@ -2333,7 +2371,7 @@ GNUNET_CHAT_message_get_discourse (const struct GNUNET_CHAT_Message *message)
 }
 
 
-const struct GNUNET_CHAT_Message*
+struct GNUNET_CHAT_Message*
 GNUNET_CHAT_message_get_target (const struct GNUNET_CHAT_Message *message)
 {
   GNUNET_CHAT_VERSION_ASSERT();
@@ -2358,22 +2396,31 @@ GNUNET_CHAT_message_get_target (const struct GNUNET_CHAT_Message *message)
 
 
 enum GNUNET_GenericReturnValue
-GNUNET_CHAT_message_delete (const struct GNUNET_CHAT_Message *message,
-			                      struct GNUNET_TIME_Relative delay)
+GNUNET_CHAT_message_delete (struct GNUNET_CHAT_Message *message,
+			                      unsigned int delay)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
   if ((!message) || (GNUNET_YES != message_has_msg(message)) || 
       (!(message->context)))
     return GNUNET_SYSERR;
+  
+  struct GNUNET_TIME_Relative rel = GNUNET_TIME_relative_multiply(
+    GNUNET_TIME_relative_get_second_(), delay
+  );
+
+  GNUNET_MESSENGER_delete_message(
+    message->context->room,
+    &(message->hash),
+    rel
+  );
 
-  GNUNET_MESSENGER_delete_message(message->context->room, &(message->hash), delay);
   return GNUNET_OK;
 }
 
 
 int
-GNUNET_CHAT_message_iterate_tags (const struct GNUNET_CHAT_Message *message,
+GNUNET_CHAT_message_iterate_tags (struct GNUNET_CHAT_Message *message,
                                   GNUNET_CHAT_MessageCallback callback,
                                   void *cls)
 {
@@ -2478,7 +2525,7 @@ GNUNET_CHAT_file_get_name (const struct GNUNET_CHAT_File *file)
 }
 
 
-const struct GNUNET_HashCode*
+const char*
 GNUNET_CHAT_file_get_hash (const struct GNUNET_CHAT_File *file)
 {
   GNUNET_CHAT_VERSION_ASSERT();
@@ -2486,7 +2533,7 @@ GNUNET_CHAT_file_get_hash (const struct GNUNET_CHAT_File *file)
   if (!file)
     return NULL;
 
-  return &(file->hash);
+  return GNUNET_h2s_full(&(file->hash));
 }
 
 
@@ -2927,7 +2974,7 @@ GNUNET_CHAT_invitation_is_rejected (const struct GNUNET_CHAT_Invitation *invitat
 }
 
 
-const struct GNUNET_ShortHashCode*
+const struct GNUNET_CHAT_DiscourseId*
 GNUNET_CHAT_discourse_get_id (const struct GNUNET_CHAT_Discourse *discourse)
 {
   GNUNET_CHAT_VERSION_ASSERT();
@@ -2999,10 +3046,9 @@ GNUNET_CHAT_discourse_close (struct GNUNET_CHAT_Discourse *discourse)
 
   msg.header.kind = GNUNET_MESSENGER_KIND_SUBSCRIBE;
 
-  GNUNET_memcpy(
-    &(msg.body.subscribe.discourse),
+  util_shorthash_from_discourse_id(
     &(discourse->id),
-    sizeof(struct GNUNET_ShortHashCode)
+    &(msg.body.subscribe.discourse)
   );
 
   msg.body.subscribe.time = GNUNET_TIME_relative_hton(GNUNET_TIME_relative_get_zero_());
@@ -3038,10 +3084,9 @@ GNUNET_CHAT_discourse_write (struct GNUNET_CHAT_Discourse *discourse,
   msg.header.kind = GNUNET_MESSENGER_KIND_TALK;
   msg.body.talk.data = GNUNET_malloc(size > max_size? max_size : size);
 
-  GNUNET_memcpy(
-    &(msg.body.talk.discourse),
+  util_shorthash_from_discourse_id(
     &(discourse->id),
-    sizeof(struct GNUNET_ShortHashCode)
+    &(msg.body.talk.discourse)
   );
 
   while (size > 0)
@@ -3078,7 +3123,7 @@ GNUNET_CHAT_discourse_get_fd (const struct GNUNET_CHAT_Discourse *discourse)
 
 
 int
-GNUNET_CHAT_discourse_iterate_contacts (const struct GNUNET_CHAT_Discourse *discourse,
+GNUNET_CHAT_discourse_iterate_contacts (struct GNUNET_CHAT_Discourse *discourse,
                                         GNUNET_CHAT_DiscourseContactCallback callback,
                                         void *cls)
 {
@@ -3087,7 +3132,7 @@ GNUNET_CHAT_discourse_iterate_contacts (const struct GNUNET_CHAT_Discourse *disc
   if (! discourse)
     return GNUNET_SYSERR;
 
-  int result = 0;
+  int iterations = 0;
 
   struct GNUNET_CHAT_DiscourseSubscription *sub;
   for (sub = discourse->head; sub; sub = sub->next)
@@ -3098,8 +3143,8 @@ GNUNET_CHAT_discourse_iterate_contacts (const struct GNUNET_CHAT_Discourse *disc
     if (callback)
       callback(cls, discourse, sub->contact);
 
-    result++;
+    iterations++;
   }
 
-  return result;
+  return iterations;
 }
