@@ -30,6 +30,7 @@
 #include <gnunet/gnunet_messenger_service.h>
 #include <gnunet/gnunet_reclaim_lib.h>
 #include <gnunet/gnunet_reclaim_service.h>
+#include <gnunet/gnunet_time_lib.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -79,7 +80,7 @@ task_handle_connection (void *cls)
   if (! handle->next)
     return;
 
-  const struct GNUNET_CHAT_Account *account = handle->next;
+  struct GNUNET_CHAT_Account *account = handle->next;
   handle->next = NULL;
 
   handle_connect(handle, account);
@@ -98,7 +99,7 @@ task_handle_disconnection (void *cls)
   if (! handle->next)
     return;
 
-  const struct GNUNET_CHAT_Account *account = handle->next;
+  struct GNUNET_CHAT_Account *account = handle->next;
   handle->next = NULL;
 
   handle_connect(handle, account);
@@ -364,7 +365,7 @@ task_group_destruction (void *cls)
 
 struct GNUNET_CHAT_GroupIterateContacts
 {
-  const struct GNUNET_CHAT_Group *group;
+  struct GNUNET_CHAT_Group *group;
   GNUNET_CHAT_GroupContactCallback cb;
   void *cls;
 };
@@ -471,7 +472,7 @@ it_context_iterate_discourses (void *cls,
 
 struct GNUNET_CHAT_MessageIterateReadReceipts
 {
-  const struct GNUNET_CHAT_Message *message;
+  struct GNUNET_CHAT_Message *message;
   GNUNET_CHAT_MessageReadReceiptCallback cb;
   void *cls;
 };
@@ -506,8 +507,12 @@ it_message_iterate_read_receipts (void *cls,
   if (!timestamp)
     return GNUNET_YES;
 
+  struct GNUNET_TIME_Absolute abs = GNUNET_TIME_absolute_ntoh(
+    it->message->msg->header.timestamp
+  );
+
   struct GNUNET_TIME_Relative delta = GNUNET_TIME_absolute_get_difference(
-    *timestamp, GNUNET_CHAT_message_get_timestamp(it->message)
+    *timestamp, abs
   );
 
   int read_receipt;
@@ -535,7 +540,7 @@ cont_update_attribute_with_status (void *cls,
 
   attributes->op = NULL;
 
-  const struct GNUNET_CHAT_Account *account = attributes->account;
+  struct GNUNET_CHAT_Account *account = attributes->account;
   struct GNUNET_CHAT_Handle *handle = attributes->handle;
 
   const char *attribute_name = NULL;
